#!/bin/sh

#
# Wrapper apt-move that lets it work as normal user, completely
# separated from the system. This makes it also possible to use it
# on non-debian systems. Most importantly, the command 'faimirror'
# is added as seen in the help message.
#
set -e

#
# Requirements for apt-move:
#   GNU sed, bc, openssl | sha1sum (sarge: coreutils), perl, ...
#   for this script additionally: find, apt-get

#####################################################################
# SETTINGS, you can change this
#

# directories

# Base directory of this package
PREFIX_BIN=`dirname $0`
if [ "$PREFIX_BIN" = "." ]; then
    PREFIX_BIN=`pwd`
fi
# Where will be repository be built?
PREFIX_DEST=$PREFIX_BIN/../debian

# configuration files
APT_CONFIG=$PREFIX_BIN/apt.conf
APTMOVE_CONFIG=$PREFIX_BIN/apt-move.conf
APTMOVE_SCRIPTS=$PREFIX_BIN/bin/share_apt-move
SOURCES_LIST=$PREFIX_BIN/sources.list
# SOURCES_LIST_GENERATED is generated by this program. It's SOURCES_LIST
# with added file:/ locations for the mirror, so only new files are
# downloaded. apt.conf's directive Etc::sourcelist must point to it too.
SOURCES_LIST_GENERATED=$PREFIX_BIN/cache/sources.generated.list
# SOURCES_LIST_CLIENT is generated by this program too, but it contains
# only file:/ locations and deb-src lines. You use it as sources.list
# for your clients. You will only be able to install packages from
# the mirror though. File is not made if this variable is empty.
# When you set this variable, SOURCES_LIST_CLIENT_PREFIX must be
# set too, that is where PREFIX_DEST is mounted on the client.
SOURCES_LIST_CLIENT=$PREFIX_DEST/sources.list
SOURCES_LIST_CLIENT_PREFIX=/schijven/.debmirror/debian

# fai package_lists directory
PKGLISTDIR=$PREFIX_BIN/pkglists

#
# END OF SETTINGS
#####################################################################

# Get all hostnames from sources.list
# Lines have to start with: deb (ftp|http)://mirror.here.net
APT_SITES=`cat $SOURCES_LIST | sed 's/^[[:space:]]*deb[[:space:]]\+\(ftp\|http\):\/\/\([-a-zA-Z0-9\.=+]\+\).*$/\2/;t;d'`

SCRIPTS=$APTMOVE_SCRIPTS    # apt-move expects it here

export SCRIPTS              # Used by apt-move
export APT_CONFIG           # Used by apt-get and apt-cache
# Variables used in apt-move.conf
export APT_SITES
export PREFIX_DEST
export PREFIX_BIN

# Run command
docommand() {
    case "$1" in
    faimirror)
        #
        # Update the fai mirror
        #
        [ -d $PKGLISTDIR/ ] || {
            echo "$PKGLISTDIR is not a valid directory. Point it to your FAI package_lists." >/dev/stderr
            exit 1
        }

        trap 'echo "
>>> User break" >/dev/stderr' EXIT

        # Make sure we have a sources.list that's up-to-date
        docommand gensources
        # Update package lists
        docommand apt -q update


        # Find out what packages we need
        for pkgfile in $PKGLISTDIR/*; do
            echo ">>> $pkgfile <<<"
            cat $pkgfile | sed 's/#.*$//; s/^[[:space:]]*PACKAGES.*$//' | \
                xargs $PREFIX_BIN/bin/apt-get install $PKGS
        done

        # Update lists
        docommand update

        echo ">>> Done! <<<"
        trap EXIT
        ;;

    clean)
        # Clean up apt-get related files
        rm -f \
            $PREFIX_BIN/lists/* \
            $PREFIX_BIN/lists/partial/* \
            $PREFIX_BIN/cache/*cache.bin \
            $SOURCES_LIST_GENERATED \
            $PREFIX_BIN/cache/lock \
            $PREFIX_BIN/cache/archives/* \
            $PREFIX_BIN/cache/archives/partial/* 2>/dev/null
        echo "It's clean now."
        ;;

    destclean)
        echo "WARNING: This will wipe all your time-consuming downloads!"
        echo "You have 3 seconds to reconsider and press ^C."
        sleep 3
        echo -n "Too late, running rm -Rf /"
        sleep 1
        echo " ... just kidding. Now removing mirror."

        rm -Rf $PREFIX_DEST/* $PREFIX_DEST/.apt-move $SOURCES_LIST_CLIENT
        ;;

    apt)
        shift
        $PREFIX_BIN/bin/apt-get $@;
        extrashift=-1   # No more cmdline args
        ;;

    filesources)
        pr=$2
        if [ ! "$pr" ]; then
            pr=$SOURCES_LIST_CLIENT
        fi
        if [ ! "$pr" ]; then
            echo "filesources command needs an argument." >/dev/stderr
        else
            extrashift=1
            find $PREFIX_DEST/.apt-move/dists -type d -name .index | sed \
               's|^.*/.apt-move/dists/\(.\+\)/\([^\/]\+\)/binary.*|deb file:'$pr' \1 \2|; s|^\(deb file:'$pr' [^- ]\+\)\-\(.*\)$|\1\/\2|' | uniq | grep -v non-US || true
        fi
        ;;

    clientsources)
       docommand filesources $SOURCES_LIST_CLIENT_PREFIX >$SOURCES_LIST_CLIENT
       echo "" >>$SOURCES_LIST_CLIENT
       cat $SOURCES_LIST | grep '^[[:space:]]*deb-src' >>$SOURCES_LIST_CLIENT
       ;;

    gensources)
        # Create sources.generated.list that also contains file:/ locations
        # pointing to the mirror created.
        docommand filesources $PREFIX_DEST >$SOURCES_LIST_GENERATED
        cat $SOURCES_LIST >>$SOURCES_LIST_GENERATED
        ;;

    *)
        # Unknown command, pass rest of command line to apt-move
        $PREFIX_BIN/bin/apt-move -c $APTMOVE_CONFIG $@
        extrashift=-1   # No more cmdline args
        ;;
    esac
}

# Show help message, duh
showusage() {
    # No args given, add commands of this script to help message
    $PREFIX_BIN/bin/apt-move -c $APTMOVE_CONFIG 2>&1 | sed \
        '/Options:/ i\
Commands added by wrapper script '`basename $0`':\
       faimirror   - update mirror from fai package lists. See this\
                     file for more information. You might need to tweak\
                     some variables before running.\
       clean       - remove all apt-related files that will be recreated\
                     automatically, like lists, archives and cache.\
       destclean   - remove whole repository mirror.\
       apt         - pass following arguments to apt-get.\
       filesources - print sources.list that can be used for the repository.\
                     The following argument is the absolute path to the mirror\
                     on your clients.\
       gensources  - creates sources.generated.list from sources.list that\
                     contains file:/ locations in addition to sources.list.\
     clientsources - use gensources to create a sources.list for the client\
                     ['$SOURCES_LIST_CLIENT'].\

      ' 1>&2
}

# Initialize repository at first run
aptmoveinit() {
    # Create sources.list without file:/ locations so apt-move can setup.
    # This is needed for gensources.
    echo ">>> Initializing mirror $PREFIX_DEST <<<"
    cp $SOURCES_LIST $SOURCES_LIST_GENERATED
    docommand apt -q=2 update
    docommand update
    docommand gensources
}

# Make sure everything's allright
configcheck() {
    # Make sure the user has setup everything correctly
    [ -r "$SOURCES_LIST" ] || {
        echo "Can't read $SOURCES_LIST, please create a valid sources.list first." >/dev/stderr
        exit 1
    }
    [ -r "$APT_CONFIG" ] || {
        echo "Can't read $APT_CONFIG, please create a valid apt.conf first." >/dev/stderr
        exit 1
    }
    [ -r "$APTMOVE_CONFIG" ] || {
        echo "Can't read $APTMOVE_CONFIG, please create a valid apt-move.conf first." >/dev/stderr
        exit 1
    }
    [ -e "$APTMOVE_SCRIPTS/move3" ] || {
        echo "$APTMOVE_SCRIPTS is not apt-move's share directory." >/dev/stderr
        exit 1
    }

    # If repository isn't setup yet, prepare
    [ -d "$PREFIX_DEST/.apt-move" ] || aptmoveinit
}

configcheck

[ $# -eq 0 ] && showusage

while [ "$1" ]; do
    extrashift=0
    docommand $@
    shift
    [ $extrashift -eq 1 ] && shift
    [ $extrashift -eq -1 ] && exit
    extrashift=0
done

exit

